;************************************************************ 
;* VFD Displaymodul 9-stellig mit ASCII Kommunikation 
;* das empfangene Zeichen wird als Echo zurckgesendet 
;* freie Software von Rudi Drabek 
;* bei Verwendung bitte (C) angeben 
;************************************************************ 
; Tiny2313 
    .include "tn2313def.inc" 
;VFD Display 
;RS232, Empfangen und Senden mit 9600 Baud bei 4 MHz 
   .def    A          = r16 
   .def    B          = r17 
   .def    zeige      = r18 
   .def    gridN     = r19 
    .def    Delay      = r20 
   .def    Count      = r21 
    .def    Count2     = r23 
    .def    Kom        = r22 
   .def    seg_code = r24 
    .def    mux        =r25    ;zeigt auf das digit, das leuchten soll 
;Port D 
   .equ   TXD    = 1    ;PortD1 hat 1k ist also Sender an  PC 
   .equ   RXD    = 0    ;PortD0 hat 100k ist Empfnger vom PC 

;used 146 words/1024 
;Xtal 4 MHz 
;Fuses auf NULL: Bodlevel1, SUT0, CKSEL1 
;Seitenangaben beziehen sich auf die Atmel Dokumentation 


.org    $0000 
       rjmp $13        ;spter in den Bootloader 
.org    $0007            ;RXCIE Interrupt, "receive complete" lst INT aus 
        rjmp RdCom 
.org     $0013 
        rjmp Anfang 

;die Tabelle fr den Inhalt des Displays: 
varlist:  .DB 0, 1, 2, 3, 4, 5, 6, 7, 8, 00 

;Zum 10er Decoder gehen die PortsD 2(LSB) 3 4 6 , also der Gridcode 
;Stelle des Displays in der Reihenfolge links ---> rechts 
.DB $10, $08, $18, $04, $14, $0c, $1c, $40, $50, 00 

;Achtung, der 7-Segmentcode ist verdrahtungsabhngig, nicht universell 
;der neue alfanumerische 7-Segmentcode 

;      0   1   2  3   4   5   6   7   8   9   bl  bl  bl  bl  bl  - 
.DB  $88,$dd,$a4,$94,$d1,$92,$83,$dc,$80,$90,$ff,$ff,$ff,$ff,$ff,$F7 

;    @      A   B   C   D   E   F   G   H   I   J   K   L   M   N   O 
.db     $84,$C0,$83,$AA,$85,$A2,$E2,$8A,$C1,$DF,$9D,$A3,$AB,$C8,$C7,$87 

;    P    Q   R  S   T   U   V   W   X   Y   Z   fill 
.db $E0,$A8,$E7,$9A,$EA,$8F,$8B,$89,$CF,$E1,$AC,  00 

.cseg 
Anfang: 
        ldi     A, Ramend 
        out        SPL,A        ;stack setzen 
        ldi        A,255        ;Datenrichtung ganzes Port ist Output 
        out     ddrb,A        
        ldi     A,0b1011110    ;PD0 = RX_RS 232 PD5 = count_in 
        out     ddrd,A 

;obige 64 Byte Tabelle (32 dW)ins Ram ab Beginn = $60 bertragen 
        clr     ZH            ;Quellpointer 
        ldi     ZL,(varlist) * 2    ;Tabellenadresse im Flash bytewise adressiert 
        clr     XH            ;Zielpointer 
        ldi     XL,charbuf    ;ab Beginn Ram+10 schreiben 
        ldi     Count,64    ;Tabelle 
table:    lpm        A,Z+        ;hole byte aus Flash und 
        st        X+,A        ;schreibe es ins RAM 
        dec     Count        ;alle 32 byte 
        brne     table 
        ldi        mux,9        ;9 Stellen des Displays mssen multiplext werden 

;RS232_init 
        cli                    ;disable Interrupt during setup of USART 
        clr     A 
        out        UBRRH,A        ;baudrate 9600 bei 4 Mhz aus Tabelle 
        ldi        A,25 
        out        UBRRL,A    
        ldi        A,0b0000110    ;8N1 setzen 
        out        UCSRC,A 
        ldi        A,0b10011000; RX und TX enable, no interrupt 
        out        UCSRB,A         ; 
        ;UCSRA UDRE =1 siehe Seite 133  bereit fr "receive new data" 
        sei                    ;enable Interrupt 
        clr        XH            ;an sich nicht ntig, aber richtig 
        clr        YH 
        clr        ZH 
        ldi        kom,1        ;fr transfer charbuf-->showchar 

Schleife: 
;Schleife langsamer machen, aber ld_display soll nicht flimmern 
        ldi       Count2, 137    ;1 
S3:     dec       Count2        ;1 ;innere Delayschleife 102,75 uS 
       brne      S3            ;2/1  137x3 = 1*(411 +5)+5 x cycletime =0,1 msec 

        rcall     ld_display    ;muss getaktet sein, sonst brennt ja nur eine Stelle 

        dec        mux            ;nchstes digit adressieren 
        brne    S4 
        ldi        mux,9        ;einen neuen digitdurchlauf starten 
S4:        rjmp      Schleife    ;2 

;multiplexte Displayroutine zur Anzeige des RAMinhalts 
;"mux"=r25 enthlt den Pointer auf die anzuzeigende Zahl 
;und ist gleichzeitig Pointer fr das digit des Displays, logisch. 
;9 stellen sind darzustellen. r16 bis r24 werden nicht gendert 
ld_display:                        ; X Y Z werden verndert 
        ldi     XL, showchar-1    ;Ausgabetab-1 laden da mux von 1-9 geht 
        add        XL, mux        ;welche Zahl soll in digit="mux" aufscheinen 
        ld        zeige, X    ;hole den Wert der Zahl aus dem RAM 
;jetzt umcodieren auf 7segmentcode 
        ldi        YL, seg7    ;offset der displayumcodierung 
        add     YL,    zeige    ;addiere welche Zahl es ist 
        ld      seg_code, Y    ;und hole den Displaycode aus der Tabelle 
        out     PortB, seg_code ;7 Segmentcode der Zahl ausgeben 

        ldi     ZL, grid-1    ;Beginn der digittabelle-1 laden 
        add        ZL,    mux        ;digitzahl fr 10er decode holen 
        ld         gridN, Z    ;und  Stelle adressieren 
        out     PortD,gridN;10er Decoder ansteuern 
        ret                    ;Dauer der Routine weniger als 10uS 


;Empfange Zeichen via Interruptsteuerung. 
;solange diese Routine tut, sind Interrupts blockiert 
RdCom:    ;zuerst die 9 digits um eine Stelle nach links 
        ldi        YL,charbuf 
        ldi        A,8            
d_shift:ldd        B,Y+1        ;hole Stelle+1, speichere 1 Stelle links davon 
        st        Y+,B        ;dann nchste Stelle rechts adressieren 
        dec        A            ;8x das ganze 
        brne    d_shift 
    ;erst wenn 9. digit frei Interrupt bedienen und digit schreiben 
        in        A,UDR        ;hole USART empfangenes Byte ab 
        rcall     WrCom        ;Echo out 

;Zeichen fr die 7-Segmenttabelle umrechnen 
        cpi        A,$2f        ;space oder "-" herausarbeiten 
        BRLO    isspace 
        cpi        A,$7b        ;  <Z ergibt space als default 
        BRSH    out_range 
        andi    A,$5F        ;in Grobuchstaben wandeln 
        cpi        A,$3A        ; 
        BRLO    isnr 
        subi    A,$30        ;Buchstaben nun im Bereich ab $10 
        rjmp    putinbuf 
isnr:    subi    A,$10        ;mit andi 5F und subi $10 in total subi $30 
        rjmp    putinbuf    ;Ziffern im Bereich 0....9 
isspace:cpi        A,$21        ;space ? 
        BRLO    is_            ;ja      " " 
        subi    A,30 
        rjmp    putinbuf    ;es ist "-" 
is_:    subi    A,18        ;mach aus " " Position 14 in der 7-Seg Tabelle 
        rjmp    putinbuf 
out_range:ldi    A,14            ;gib space aus 

putinbuf:ldi    YL,charbuf+8;und 
        st        Y,A            ;ins Display rechtsbndig = 9.digit schreiben 
;        inc        kom            ;zhlt die empfangenen bytes 
;        cpi        kom,9 
;        breq    transbuf    ;wenn 'L' 'C' bis an digit1 gelangt ist, dann 
                            ;charbuf --> showchar speichern 
;notrans:cpi        A,10        ;ist es nibble 'z' 
;        brne    weg 
;        ldi        kom,1 
;        rjmp    weg 

;transfer charbuf --> showchar 
transbuf:push    A 
        push    B 
        ldi        B,9    ;9 bytes/digits from receive buffer to showbuffer 
        ldi        XL,charbuf    ;von hier: Quelle charbuf 
        ldi        ZL,showchar    ;nach da, damit Display nicht flackert 
cpybuf:    ld        A,X+ 
        st        Z+,A 
        dec        B            ; 
        brne    cpybuf 
        pop     B 
        pop        A 
;        rjmp    notrans        ;showbuf jetzt nur nach 9 bytes zeigen 
weg:    reti                ;nicht vergessen  "i" 


WrCOM:  sbis    UCSRA,UDRE    ;wait for empty transmit buffer 
        rjmp    WrCom 
        out        UDR,A        ;sende Byte in A 
        ret 

.dseg 
showchar:     .byte  10 
charbuf:    .byte    10 
grid:        .byte    10 
seg7:        .byte    44 
